/*
 * QSDEInterface.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on May 30, 2007, 1:55 PM
 *
 */

package info.frantzen.testing.ssmsimulator;

import info.frantzen.testing.ssmsimulator.ssm.InstantiatedMessage;
import info.frantzen.testing.ssmsimulator.ssm.MessageKind;
import info.frantzen.testing.ssmsimulator.ssm.OperationKind;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

/**
 *
 * @author Lars Frantzen
 */
public class QSDEInterface {
    
    private Socket soc;
    private PrintWriter out = null;
    private BufferedReader in = null;
    private String inputString;
    
    
    /** Creates a new instance of QSDEInterface */
    public QSDEInterface(Socket solverSocket, Logger logger) throws IOException {
        soc = solverSocket;
        out = new PrintWriter(soc.getOutputStream(), true);
        in = new BufferedReader(new InputStreamReader(soc.getInputStream()));
    }
    
    public void initialiseSequenceDiagram(String name) {
        out.println(name);
        out.println("a:" + name + "[a]\ni:SUT[a]\n");
    }
    
    public void updateSequenceDiagram(InstantiatedMessage ime) {
        StringBuffer sb = new StringBuffer();
        
        if ((ime.getMessage().getKind() == MessageKind.INPUT) && ime.getMessage().getOperation().getKind() == OperationKind.REQUESTRESPONSE) {
            inputString = ime.toString();
            return;
        }
        else if ((ime.getMessage().getKind() == MessageKind.OUTPUT) && (ime.getMessage().getOperation() == null))
            sb.append("[c:quiescence]\na:\n[/c]");
        else if ((ime.getMessage().getKind() == MessageKind.OUTPUT) && ime.getMessage().getOperation().getKind() == OperationKind.REQUESTRESPONSE) {
            sb.append("a:");
            sb.append(ime.toString());
            sb.append("=i.");
            sb.append(inputString);
//            sb.append('\n');
        } else if ((ime.getMessage().getKind() == MessageKind.INPUT) && ime.getMessage().getOperation().getKind() == OperationKind.ONEWAY) {
            sb.append("a:>i.");
            sb.append(ime.toString());
            sb.append("\ni:stop");
//            sb.append('\n');
        } else {
            System.err.println("Only Request-Response and One-Way supported by the QSDEInterface!");
            System.exit(1);
        }
        out.println(sb.toString());
    }
    
    public static String suspensionTraceAsQSDE(List<InstantiatedMessage> suspensionTrace) {
        StringBuffer sb = new StringBuffer();
        sb.append("a:Jambition[a]\ni:SUT[a]\n\n");
        for (Iterator it = suspensionTrace.iterator(); it.hasNext();) {
            InstantiatedMessage ime = (InstantiatedMessage) it.next();
            if ((ime.getMessage().getKind() == MessageKind.INPUT) && ime.getMessage().getOperation().getKind() == OperationKind.REQUESTRESPONSE) {
                InstantiatedMessage ret = (InstantiatedMessage) it.next();
                sb.append("a:");
                sb.append(ret.toString());
                sb.append("=i.");
                sb.append(ime.toString());
                sb.append('\n');
            } else if ((ime.getMessage().getKind() == MessageKind.INPUT) && ime.getMessage().getOperation().getKind() == OperationKind.ONEWAY) {
                sb.append("a:>i.");
                sb.append(ime.toString());
                sb.append("\ni:stop");
                sb.append('\n');
            } else if ((ime.getMessage().getKind() == MessageKind.OUTPUT) && (ime.getMessage().getOperation() == null))
                sb.append("[c:quiescence]\na:\n[/c]\n");
            else {
                System.err.println("Only Request-Response and One-Way supported the QSDEInterface!");
                System.exit(1);
            }
        }
        return sb.toString();
    }
    
}
