/*
 * MyNode.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on December 22, 2006, 10:33 AM
 *
 */

package info.frantzen.testing.ssmsimulator.interpreter;

import java.util.Hashtable;

/**
 * This is the main node class. It just serves to to add some features to the real nodes in the javacc
 * package which subclass this class (via subclassing SimpleNode).
 * @author Lars Frantzen
 * @version 0.1
 */
public class MyNode {
    
    /**
     * Static stack for calculations. Since we don't have a static parser this is not thread-safe.
     * Not sure if this stack is really needed if the interpret method would return the values it now
     * puts on the stack. Then, this should also be thread-safe.
     */
    public static Object[] stack = new Object[1024];
    /**
     * The top of the stack.
     */
    public static int top = -1;
    /**
     * The type of the node. A type is a {@link info.frantzen.testing.ssmsimulator.interpreter.javacc.DumontParserConstants Dumont parser constant}.
     */
    public int type;
    
    /**
     * In this hashtable the relation of the Prolog variable names and the SSM variable names is stored
    */
    public static Hashtable<String,String> prologVartab = new Hashtable<String,String>();
    
    public static int prologVarCounter = 0;
    
    public static String getNextPrologVarName() {
        return 'X' + Integer.toString(prologVarCounter++);
    }
    
    /**
     * The interpret method of the node. Every real node must implement this method, so that it never reaches this
     * implementation (leading to an exception).
     * @throws ssmsimulator.interpreter.javacc.DumontParser.RuntimeViolation If a real node is interpreted which does not implement the interpret() method, this
     * exception is thrown.
     */
    public void interpret() throws info.frantzen.testing.ssmsimulator.interpreter.javacc.DumontParser.RuntimeViolation {
        throw new UnsupportedOperationException(); // It better not come here.
    }
    
    /**
     * The static type check method of the node. Every real node must implement this method, so that it never reaches this
     * implementation (leading to an exception).
     * @return Here nothing, real nodes return their type.
     * @throws ssmsimulator.interpreter.javacc.DumontParser.IllegalTypeException If a real node is interpreted which does not implement the typeCheck() method, this
     * exception is thrown.
     */
    public int typeCheck() throws info.frantzen.testing.ssmsimulator.interpreter.javacc.DumontParser.IllegalTypeException {
        throw new UnsupportedOperationException(); // It better not come here.
    }
    
    /** The Prolog query generator. Every real node must implement this method, so that it never reaches this
     * implementation (leading to an exception).
     **/
    public String getPrologQuery() throws info.frantzen.testing.ssmsimulator.interpreter.javacc.DumontParser.RuntimeViolation {
        throw new UnsupportedOperationException(); // It better not come here.
    }
}
