/*
 * ST_Double.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on 30. Dezember 2006, 13:30
 *
 */

package info.frantzen.testing.ssmsimulator.types;

import javax.xml.namespace.QName;

/**
 * The (simple) Double type.
 * @author Lars Frantzen
 * @version 0.3
 */
public class ST_Double extends SimpleType {
    
    /**
     * Creates a new instance of ST_Double.
     */
    public ST_Double() {
        super(new QName("ST_Double"), "Double");
    }
    
    /**
     * Returns a random {@link ST_DoubleInstance ST_Double instance}.
     * @return a random ST_Double instance
     */
    public ST_DoubleInstance generateRandomInstance() {
        return new ST_DoubleInstance(new Double(random.nextInt() + random.nextDouble()));
    }
    
    /**
     * Returns the next boundary {@link ST_DoubleInstance ST_Double instance}.<br>
     * <B>Warning: </B>not yet implemented correctly!
     * @return the next boundary ST_Double instance
     */
    public ST_DoubleInstance generateNextBoundaryInstance() {
        // TODO
        return new ST_DoubleInstance(new Double(42.));
    }
    
    /**
     * Returns the number of boundary {@link ST_DoubleInstance ST_Double instances}.<br>
     * <B>Warning: </B>not yet implemented correctly!
     * @return the number of boundary ST_Double instances
     */
    public int numberOfBoundaryInstances() {
        // TODO
        return 42;
    }
    
    /**
     * Returns the initial {@link ST_DoubleInstance ST_Double instance}, i.e. 0.
     * @return the initial ST_Double instance, i.e. 0.
     */
    public ST_DoubleInstance generateInitialInstance() {
        return new ST_DoubleInstance(new Double(0.));
    }
    
    /**
     * Returns true for all Double objects since this simple type is not restricted.
     * @param o the object for which the restriction shall be verified
     * @return true for all Double instances
     */
    public boolean checkRestriction(Object o) {
        if (o instanceof Double)
            return true;
        else
            return false;
    }
    
    /**
     * A string representation of the type.
     * @return a string representation of the type
     */
    public String toString() {
        return "ST_Double";
    }
    
    
}
