/*
 * ST_PosDouble.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on 30. Dezember 2006, 13:30
 *
 */

package info.frantzen.testing.ssmsimulator.types;

import javax.xml.namespace.QName;

/**
 * The (simple) Double type with a restriction on non-negative values.
 * @author Lars Frantzen
 * @version 0.3
 */
public class ST_PosDouble extends SimpleType {
    
    /**
     * Creates a new instance of ST_PosDouble.
     */
    public ST_PosDouble() {
        super(new QName("ST_PosDouble"), "Double");
    }
    
    /**
     * Returns a random {@link ST_PosDoubleInstance ST_PosDouble instance}.
     * @return a random ST_PosDouble instance
     */
    public ST_PosDoubleInstance generateRandomInstance() {
        return new ST_PosDoubleInstance(new Double(Math.abs(random.nextInt() + random.nextDouble())));
    }
    
    /**
     * Returns the next boundary {@link ST_PosDoubleInstance ST_PosDouble instance}.<br>
     * <B>Warning: </B>not yet implemented correctly!
     * @return the next boundary ST_PosDouble instance
     */
    public ST_PosDoubleInstance generateNextBoundaryInstance() {
        // TODO
        return new ST_PosDoubleInstance(new Double(42.));
    }
    
    /**
     * Returns the number of boundary {@link ST_PosDoubleInstance ST_PosDouble instances}.<br>
     * <B>Warning: </B>not yet implemented correctly!
     * @return the number of boundary ST_PosDouble instances
     */
    public int numberOfBoundaryInstances() {
        // TODO
        return 42;
    }
    
    /**
     * Returns the initial {@link ST_PosDoubleInstance ST_PosDouble instance}, i.e. 0.
     * @return the initial ST_PosDoubleInt instance, i.e. 0.
     */
    public ST_PosDoubleInstance generateInitialInstance() {
        return new ST_PosDoubleInstance(new Double(0.));
    }
    
    /**
     * Returns true for all Double instances with a value >= 0.
     * @param o the object for which the restriction shall be verified
     * @return true for all Double instances with a value >= 0., false otherwise
     */
    public boolean checkRestriction(Object o) {
        if (o instanceof Double)
            return ((Double) o) >= 0;
        else
            return false;
    }
    
    /**
     * A string representation of the type.
     * @return a string representation of the type
     */
    public String toString() {
        return "ST_PosDouble";
    }
    
     
    
}
