/*
 * ST_PseudoPosDouble.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on May 6 2007
 *
 */

package info.frantzen.testing.ssmsimulator.types;

import info.frantzen.testing.ssmsimulator.TreeSolverInterface;
import javax.xml.namespace.QName;

/**
 * The pseudo (simple) Double type with a restriction on non-negative values.
 * @author Lars Frantzen
 * @version 0.3
 */
public class ST_PseudoPosDouble extends SimpleType {
    
    public static int postPointLength = 1;
    
    /**
     * Creates a new instance of ST_PseudoPosDouble. postPointLength must be >= 1
     */
    public ST_PseudoPosDouble() {
        super(new QName("ST_PosDouble"), "Double");
    }
    
    /**
     * Returns a random {@link ST_PseudoPosDoubleInstance ST_PseudoPosDouble instance}.
     * @return a random ST_PseudoPosDouble instance
     */
    public ST_PseudoPosDoubleInstance generateRandomInstance() {
        int prePointValue = random.nextInt(TreeSolverInterface.MAXINTRANDOM);
        int postPointValue = random.nextInt((int) Math.pow(10,postPointLength));
        return new ST_PseudoPosDoubleInstance(prePointValue, postPointValue);
    }
    
    /**
     * Returns the next boundary {@link ST_PseudoPosDoubleInstance ST_PseudoPosDouble instance}.<br>
     * <B>Warning: </B>not yet implemented correctly!
     * @return the next boundary ST_PseudoPosDouble instance
     */
    public ST_PseudoPosDoubleInstance generateNextBoundaryInstance() {
        // TODO
        return new ST_PseudoPosDoubleInstance(42, 0);
    }
    
    /**
     * Returns the number of boundary {@link ST_PseudoPosDoubleInstance ST_PseudoPosDouble instances}.<br>
     * <B>Warning: </B>not yet implemented correctly!
     * @return the number of boundary ST_PseudoPosDouble instances
     */
    public int numberOfBoundaryInstances() {
        // TODO
        return 42;
    }
    
    /**
     * Returns the initial {@link ST_PseudoPosDoubleInstance ST_PseudoPosDouble instance}, i.e. 0.0
     * @return the initial ST_PseudoPosDoubleInt instance, i.e. 0.
     */
    public ST_PseudoPosDoubleInstance generateInitialInstance() {
        return new ST_PseudoPosDoubleInstance(0, 0);
    }
    
    /**
     * @param o the object for which the restriction shall be verified
     */
    public boolean checkRestriction(Object o) {
        if (o instanceof Double) {
            Double d = (Double) o;
            if (d < 0)
                return false;
            String s = d.toString();
            return (s.substring(s.indexOf('.')+1).length() <= postPointLength);
        } else
            return false;
    }
    
    /**
     * A string representation of the type.
     * @return a string representation of the type
     */
    public String toString() {
        return "ST_PseudoPosDouble";
    }
    
    public static ST_PseudoPosDoubleInstance instanceFromNumber(Integer num) {
        int power = (int) Math.pow(10,postPointLength);
        int postPointValue = num % power;
        int prePointValue = (num - postPointValue) / power;
        return new ST_PseudoPosDoubleInstance(prePointValue, postPointValue);
    }
    
    public static int stringAsPostInt(String s) {
        for (int i = 0; i < ST_PseudoPosDouble.postPointLength - s.length(); i++)
            s = s + '0';
        return Integer.valueOf(s);
    }
    
    public static String postIntAsString(int j) {
        String s = Integer.toString(j);
        for (int i = 0; i < ST_PseudoPosDouble.postPointLength - s.length(); i++)
            s = '0' + s;
        return s;
    }
    
    
}
