/*
 * ST_String.java
 * Copyright (C) 2007 Lars Frantzen
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * See the LICENSE file for the full license
 *
 * Created on 30. Dezember 2006, 13:35
 *
 */

package info.frantzen.testing.ssmsimulator.types;

import java.util.Hashtable;
import java.util.Set;
import javax.xml.namespace.QName;

/**
 * The (simple) String type.
 * @author Lars Frantzen
 * @version 0.3
 */
public class ST_String extends SimpleType {
    
    static private Hashtable<String, Integer> getNumberTab = new Hashtable<String, Integer>();
    static private Hashtable<Integer, String> getStringTab = new Hashtable<Integer,String>();
    
    static private int nextStringNumber;
    static private int nextRandomStringNumber;
    
    /**
     * Creates a new instance of ST_String.
     */
    public ST_String() {
        super(new QName("ST_String"), "String");
    }
    
    /**
     * Returns a random {@link ST_StringInstance ST_String instance}.<br>
     * <B>Warning: </B>not yet implemented correctly!
     * @return a random ST_String instance
     */
    public ST_StringInstance generateRandomInstance() {
        return new ST_StringInstance("NOT YET IMPLEMENTED");
    }
    
    static public ST_StringInstance generateStaticRandomInstance() {
        return new ST_StringInstance("SSMSimulator" + Integer.toString(nextRandomStringNumber++));
    }
    
    /**
     * Returns the next boundary {@link ST_StringInstance ST_String instance}.<br>
     * <B>Warning: </B>not yet implemented correctly!
     * @return the next boundary ST_String instance
     */
    public ST_StringInstance generateNextBoundaryInstance() {
        // TODO
        return new ST_StringInstance("NOT YET IMPLEMENTED");
    }
    
    /**
     * Returns the number of boundary {@link ST_StringInstance ST_String instances}.<br>
     * <B>Warning: </B>not yet implemented correctly!
     * @return the number of boundary ST_String instances
     */
    public int numberOfBoundaryInstances() {
        // TODO
        return 42;
    }
    
    /**
     * Returns the initial {@link ST_StringInstance ST_String instance}, i.e. the empty string "".
     * @return the initial ST_String instance, i.e. the empty string ""
     */
    public ST_StringInstance generateInitialInstance() {
        return new ST_StringInstance("");
    }
    
    /**
     * Returns true for all String objects since this simple type is not restricted.
     * @param o the object for which the restriction shall be verified
     * @return true for all String objects, false otherwise
     */
    public boolean checkRestriction(Object o) {
        if (o instanceof String)
            return true;
        else
            return false;
    }
    
    /**
     * A string representation of the type.
     * @return a string representation of the type
     */
    public String toString() {
        return "ST_String";
    }
    
    
    public static int instanceAsNumber(ST_StringInstance instance) {
        // String.hashCode() would be also an option, but will probably generate too big integers
        String s = instance.toString();
        
        if (! getNumberTab.containsKey(s)) {
            getNumberTab.put(s, nextStringNumber);
            getStringTab.put(nextStringNumber, s);
            return nextStringNumber++;
        } else
            return getNumberTab.get(s);
    }
    
    public static ST_StringInstance instanceFromNumber(Integer i) {
        Object o = getStringTab.get(i);
        if (o != null)
            return new ST_StringInstance((String) o);
        else {
            // This is the case where an interaction variable
            // is used in a switch restriction in a conditional manner, like: s == "foo" ==> ...
            // If the solver decides s != "foo" (which makes the restriction true), then we have to generate
            // a string which does not equal "foo". This happens here.
            Set<String> stringSet = getNumberTab.keySet();
            String candidate = generateStaticRandomInstance().toString();
            while (stringSet.contains(candidate))
                candidate = generateStaticRandomInstance().toString();
            return new ST_StringInstance(candidate);
        }
    }
    
    public static void resetInstanceNumberHandling() {
        nextRandomStringNumber = 0;
        nextStringNumber = 0;
        getNumberTab.clear();
        getStringTab.clear();
    }
    
}
